@@ -10,18 +10,19 @@ a24 (int a)
   const int c = 1;
   int i = 0;
   int l = 0;
-#pragma omp parallel default(none) private(a) shared(z)
+#pragma omp parallel default(none) private(a) shared(z, c) /* { dg-line omp_parallel } */
   {
     int j = omp_get_num_threads ();
-    /* O.K. - j is declared within parallel region */
-    /* O.K.  -  a is listed in private clause */
-    /*       -  z is listed in shared clause */
+	/* O.K. - j is declared within parallel region */
+    a = z[j]; /* O.K.  -  a is listed in private clause */
+	      /*       -  z is listed in shared clause */
     x = c;			/* O.K.  -  x is threadprivate */
-    				/*       -  c has const-qualified type */
+    				/*       -  c has const-qualified type and
+					      is listed in shared clause */
     z[i] = y;
     /* { dg-error "'i' not specified" "" { target *-*-* } .-1 } */
-    /* { dg-error "enclosing 'parallel'" "" { target *-*-* } 13 } */
-    /* { dg-error "'y' not specified" "" { target *-*-* } 21 }  */
+    /* { dg-error "enclosing 'parallel'" "" { target *-*-* } omp_parallel } */
+    /* { dg-error "'y' not specified" "" { target *-*-* } .-3 }  */
 #pragma omp for firstprivate(y)
     for (i = 0; i < 10; i++)
       {
