@@ -6,9 +6,9 @@
 int
 foo (void)
 {
-  int a[N], b[N], c[N];
+  int a[N], c[N];
   int *ap = &a[0];
-  int *bp = &b[0];
+  int *bp = &a[0];
   int *cp = &c[0];
 
 #pragma omp parallel for
@@ -22,8 +22,7 @@ foo (void)
   return *cp;
 }
 
-/* { dg-final { scan-tree-dump-times "\\] = 1;" 2 "optimized" } } */
+/* { dg-final { scan-tree-dump-times "\\] = 1;" 1 "optimized" } } */
 /* { dg-final { scan-tree-dump-times "\\] = 2;" 1 "optimized" } } */
-/* { dg-final { scan-tree-dump-times "\\] = _\[0-9\]*;" 0 "optimized" } } */
+/* { dg-final { scan-tree-dump-times "\\] = _\[0-9\]*;" 1 "optimized" } } */
 /* { dg-final { scan-tree-dump-times "\\] = " 3 "optimized" } } */
-
