@@ -12,4 +12,4 @@ var c [1.5]int	// ERROR "truncated"
 var d ["abc"]int	// ERROR "invalid array bound|not numeric"
 var e [nil]int	// ERROR "invalid array bound|not numeric"
 var f [e]int	// ERROR "invalid array bound|not constant"
-var g [1<<65]int	// ERROR "overflows"
+var g [1<<65]int	// ERROR "array bound is too large|overflows"
