@@ -1,11 +1,11 @@
 // 2004-04-30  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2004 Free Software Foundation
+// Copyright (C) 2004-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,18 +14,11 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 22.2.2.2.1  num_put members
 
-// On Solaris 10 x86, this test crashes in libc.  Inside libstdc++, 
-// we call sprintf like so:
-//   sprintf (buffer, "%.*f", 1000, 1.0)
-// which crashes.
-// { dg-do run { xfail { i?86*-*-solaris2.10 } } } 
-
 #include <locale>
 #include <sstream>
 #include <testsuite_hooks.h>
@@ -34,7 +27,6 @@
 void test01()
 {
   using namespace std;
-  bool test __attribute__((unused)) = true;
 
   ostringstream oss;
   const num_put<char>& np = use_facet<num_put<char> >(oss.getloc());
