@@ -1,9 +1,10 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
+// { dg-require-effective-target random_device }
 // { dg-require-cstdint "" }
 //
 // 2008-11-24  Edward M. Smith-Rowland <3dw4rd@verizon.net>
 //
-// Copyright (C) 2008-2015 Free Software Foundation, Inc.
+// Copyright (C) 2008-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -20,8 +21,7 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// 26.4.6 class random_device [rand.device]
-// 26.4.2.2 Concept RandomNumberEngine [rand.concept.eng]
+// C++11 26.5.6 class random_device [rand.device]
 
 #include <random>
 #include <testsuite_hooks.h>
@@ -29,12 +29,13 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
   std::random_device x;
 
-  VERIFY( x.min() == std::numeric_limits<std::random_device::result_type>::min() );
-  VERIFY( x.max() == std::numeric_limits<std::random_device::result_type>::max() );
+  using result_type = std::random_device::result_type;
+  VERIFY( x.min() == std::numeric_limits<result_type>::min() );
+  VERIFY( x.max() == std::numeric_limits<result_type>::max() );
+
+  result_type n [[gnu::unused]] = x();
 }
 
 int main()
