@@ -1,6 +1,6 @@
-// { dg-require-namedlocale "en_US.ISO8859-1" }
+// { dg-require-namedlocale "is_IS.UTF-8" }
 
-// Copyright (C) 2003-2017 Free Software Foundation, Inc.
+// Copyright (C) 2003-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -23,38 +23,26 @@
 #include <locale>
 #include <testsuite_hooks.h>
 
-class Buf2 : public std::streambuf
+class Buf1 : public std::streambuf
 {
-public:
-  std::locale before;
-  std::locale after;
-
 protected:
-  void imbue(const std::locale& loc)
-  {
-    before = getloc();
-
-    std::streambuf::imbue(loc);
-
-    after = getloc();
-  }
+  void imbue(const std::locale&)
+  { }
 };
 
 // libstdc++/13007
-void test02()
+void test01()
 {
-  Buf2 buf;
-  std::locale loc(std::locale(ISO_8859(1,en_US)));
+  Buf1 buf;
+  std::locale loc(std::locale("is_IS.UTF-8"));
 
   buf.pubimbue(loc);
 
   VERIFY( buf.getloc() == loc );
-  VERIFY( buf.before == std::locale::classic() );
-  VERIFY( buf.after == std::locale::classic() );
 }
 
 int main()
 {
-  test02();
+  test01();
   return 0;
 }
