@@ -1,6 +1,3 @@
-// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
-// reminder from Petur Runolfsson <peturr02@ru.is>.
-
 // Copyright (C) 2003-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -21,8 +18,8 @@
 // { dg-require-fileio "" }
 
 #include <iostream>
-#include <cwchar>
 #include <cstdio>
+#include <cwchar>
 #include <testsuite_hooks.h>
 
 void
@@ -30,12 +27,13 @@ test01()
 {
   VERIFY( std::freopen("cin_unget-1.txt", "r", stdin) );
 
-  wchar_t buf[2];
-  VERIFY( std::wcin.rdbuf()->sgetn(buf, 2) == 2 );
-  std::wint_t c1 = std::wcin.rdbuf()->sungetc();
-  std::wint_t c2 = std::wcin.rdbuf()->sbumpc();
-  VERIFY( c1 == std::char_traits<wchar_t>::to_int_type(buf[1]) );
-  VERIFY( c2 == c1 );
+  wchar_t c1;
+  std::wint_t c2;
+  std::wcin.get(c1);
+  std::wcin.unget();
+  VERIFY( std::wcin.good() );
+  c2 = std::fgetwc(stdin);
+  VERIFY( c2 == std::char_traits<wchar_t>::to_int_type(c1) );
 }
 
 int main(void)
