@@ -1,9 +1,8 @@
-// { dg-do compile }
-// { dg-options "-std=gnu++0x" }
+// { dg-do compile { target c++11 } }
 
 // 2007-03-12  Stephen M. Webb  <stephen.webb@bregmasoft.com>
 //
-// Copyright (C) 2010 Free Software Foundation, Inc.
+// Copyright (C) 2010-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -23,12 +22,10 @@
 // [28.8.2] class template basic_regex constructor
 
 #include <regex>
-#include <testsuite_hooks.h>
 
 // Tests C-style null-terminated-string constructor of the basic_regex class.  
 void test01()
 {
-  bool test __attribute__((unused)) = true;
   typedef std::basic_regex<wchar_t> test_type;
 
   const wchar_t* cs = L"aab";
@@ -40,4 +37,4 @@ main()
 { 
   test01();
   return 0;
-};
+}
