@@ -1,10 +1,9 @@
-// { dg-do compile }
-// { dg-options "-std=gnu++11" }
+// { dg-do compile { target c++11 } }
 
 //
 // 2010-06-10  Stephen M. Webb <stephen.webb@bregmasoft.ca>
 //
-// Copyright (C) 2010-2016 Free Software Foundation, Inc.
+// Copyright (C) 2010-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -28,7 +27,6 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
   typedef std::match_results<char*> mr;
 
   typedef mr::value_type       value_type;
@@ -41,4 +39,11 @@ test01()
   typedef mr::allocator_type   allocator_type;
   typedef mr::char_type        char_type;
   typedef mr::string_type      string_type;
+
+  static_assert(std::is_same<value_type, std::sub_match<char*>>::value, "");
+  static_assert(std::is_same<const_reference, const value_type&>::value, "");
+  static_assert(std::is_same<reference, value_type&>::value, "DR 2306");
+  static_assert(std::is_same<const_iterator, iterator>::value, "");
+  static_assert(std::is_same<char_type, char>::value, "");
+  static_assert(std::is_same<string_type, std::string>::value, "");
 }
