@@ -27,13 +27,8 @@ int main()
   using namespace __gnu_test;
 
  // cplus-dem FAIL
-  // Equivalent as considered order-insensitive?
-  // standard text, icc FAIL
-  //  verify_demangle("_ZlsRKU3fooU4bart1XS0_", 
-  //     "operator<<(X const foo bart&, X const foo bart)");
-  // new __cxa_demangle
-  verify_demangle("_ZlsRKU3fooU4bart1XS0_", 
-		  "operator<<(X bart foo const&, X bart)");
+  verify_demangle("_Z3fooA30_A_i", "foo(int [30][])");
+
 
   return 0;
 }
