@@ -1,9 +1,9 @@
-// { dg-options "-std=c++11" }
+// { dg-do run { target c++11 } }
 // { dg-require-cstdint "" }
 //
 // 2012-09-24  Edward M. Smith-Rowland  <3dw4rd@verizon.net>
 //
-// Copyright (C) 2012-2013 Free Software Foundation, Inc.
+// Copyright (C) 2012-2020 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -20,8 +20,8 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// 26.4.8.3.* Class template k_distribution [rand.dist.ext.k]
-// 26.4.2.4 Concept RandomNumberDistribution [rand.concept.dist]
+// Class template k_distribution
+// 26.5.1.6 Random number distribution requirements [rand.req.dist]
 
 #include <ext/random>
 #include <sstream>
@@ -30,8 +30,6 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
   std::stringstream str;
   __gnu_cxx::k_distribution<double> u(2.0, 1.5, 3.0), v;
   std::minstd_rand0 rng;
